#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "usesdp.h"
#include "declarations.h"

int usesdpwithg(int f1size, int f2size, int g1size, int g2size, int mem_up_bound_ci,
	int varsize, int ** myconstraintsindex, int * myconstraints, double * myconstraintsvalue, struct blockmatrix * X) {

	int fsize = f1size + f2size;
	int gsize = g1size + g2size;
	int allsize = fsize + gsize;
	int conssize = mem_up_bound_ci;

	/*
	*
	*/
	struct blockmatrix C;
	double *b;
	struct constraintmatrix *constraints;

	struct blockmatrix Z;
	double *y;
	double pobj, dobj;

	struct sparseblock *blockptr;

	int ret;

	C.nblocks = 2;
	C.blocks = (struct blockrec *)malloc(4 * sizeof(struct blockrec));
	if (C.blocks == NULL)
	{
		printf("Couldn't allocate storage for C!\n");
		exit(1);
	};

	C.blocks[1].blockcategory = DIAG;
	C.blocks[1].blocksize = allsize + 1;
	//printf("C.blocks[1].blocksize = %d\n", C.blocks[1].blocksize);
	C.blocks[1].data.vec = (double *)malloc((allsize + 3)* sizeof(double));
	if (C.blocks[1].data.vec == NULL)
	{
		printf("Couldn't allocate storage for C!\n");
		exit(1);
	};

	/*
	* Put the entries into the first block.
	*/
	for (int i = 1; i <= allsize + 1; i++){
		C.blocks[1].data.vec[i] = 0.0;
	}


	/*
	* Setup the second block.
	*/

	C.blocks[2].blockcategory = MATRIX;
	C.blocks[2].blocksize = varsize + 1;
	C.blocks[2].data.mat = (double *)malloc((varsize + 2) * (varsize + 2) * sizeof(double));
	if (C.blocks[2].data.mat == NULL)
	{
		printf("Couldn't allocate storage for C!\n");
		exit(1);
	};

	/*
	* Put the entries into the second block.
	*/

	for (int i = 1; i <= varsize + 1; i++){
		for (int j = 1; j <= varsize + 1; j++){
			C.blocks[2].data.mat[ijtok(i, j, (varsize + 1))] = 0.0;
		}
	}

	b = (double *)malloc((conssize + 3)*sizeof(double));
	if (b == NULL)
	{
		printf("Failed to allocate storage for a!\n");
		exit(1);
	};



	/*
	* Fill in the entries in b.
	*/

	for (int i = 0; i <= conssize; i++){
		b[i] = 0.0;
	}
	b[conssize + 1] = 1.0;

	constraints = (struct constraintmatrix *)malloc((conssize + 3)*sizeof(struct constraintmatrix));
	if (constraints == NULL)
	{
		printf("Failed to allocate storage for constraints!\n");
		exit(1);
	};

	/*
	* Setup the A1 matrix.  Note that we start with block 3 of A1 and then
	* do block 1 of A1.  We do this in this order because the blocks will
	* be inserted into the linked list of A1 blocks in reverse order.
	*/

	/*
	* Terminate the linked list with a NULL pointer.
	*/

	for (int i = 1; i <= conssize; i++){

		constraints[i].blocks = NULL;

		blockptr = (struct sparseblock *)malloc(sizeof(struct sparseblock));
		if (blockptr == NULL)
		{
			printf("Allocation of constraint block failed!\n");
			exit(1);
		};

		/*
		* Initialize the coefficients of M
		*/

		blockptr->blocknum = 2;
		blockptr->blocksize = varsize + 1;
		blockptr->constraintnum = i;
		blockptr->next = NULL;
		blockptr->nextbyblock = NULL;

		int blocksize = 1;

		blockptr->entries = (double *)malloc((blocksize + 1)*sizeof(double));
		if (blockptr->entries == NULL)
		{
			printf("Allocation of constraint block failed!\n");
			exit(1);
		};
		blockptr->iindices = (unsigned short *)malloc((blocksize + 2)*sizeof(unsigned short));
		if (blockptr->iindices == NULL)
		{
			printf("Allocation of constraint block failed!\n");
			exit(1);
		};
		blockptr->iindices[0] = (unsigned short)0;
		blockptr->jindices = (unsigned short *)malloc((blocksize + 2)*sizeof(unsigned short));
		if (blockptr->jindices == NULL)
		{
			printf("Allocation of constraint block failed!\n");
			exit(1);
		};
		blockptr->jindices[0] = (unsigned short)0;

		blockptr->numentries = blocksize;

		blockptr->iindices[1] = (unsigned short)myconstraintsindex[i][0];
		blockptr->jindices[1] = (unsigned short)myconstraintsindex[i][1];
		if (myconstraintsindex[i][0] == myconstraintsindex[i][1])
			blockptr->entries[1] = (double)1;
		else
			blockptr->entries[1] = 0.5;

		blockptr->next = constraints[i].blocks;
		constraints[i].blocks = blockptr;

		/*
		now, we handle the coefficient of lamda
		*/
		if (myconstraintsindex[i][2] == 0 && i != 1){
			continue;
		}

		blockptr = (struct sparseblock *)malloc(sizeof(struct sparseblock));
		if (blockptr == NULL)
		{
			printf("Allocation of constraint block failed!\n");
			exit(1);
		};

		/*
		* Initialize block 1.
		*/

		blockptr->blocknum = 1;
		blockptr->blocksize = allsize + 1;
		blockptr->constraintnum = i;
		blockptr->next = NULL;
		blockptr->nextbyblock = NULL;
		blockptr->entries = (double *)malloc((myconstraintsindex[i][2] + 3)*sizeof(double));
		if (blockptr->entries == NULL)
		{
			printf("Allocation of constraint block failed!\n");
			exit(1);
		};
		blockptr->iindices = (unsigned short *)malloc((myconstraintsindex[i][2] + 3)*sizeof(unsigned short));
		if (blockptr->iindices == NULL)
		{
			printf("Allocation of constraint block failed!\n");
			exit(1);
		};
		blockptr->iindices[0] = (unsigned short)0;
		blockptr->jindices = (unsigned short *)malloc((myconstraintsindex[i][2] + 3)*sizeof(unsigned short));
		if (blockptr->jindices == NULL)
		{
			printf("Allocation of constraint block failed!\n");
			exit(1);
		};
		blockptr->jindices[0] = (unsigned short)0;

		if (myconstraintsindex[i][0] == 1 && myconstraintsindex[i][1] == 1){
			blockptr->numentries = myconstraintsindex[i][2] + 1;
			for (int iter = 1; iter <= myconstraintsindex[i][2]; iter++){
				blockptr->iindices[iter] = (unsigned short)myconstraints[myconstraintsindex[i][iter + 2]];
				blockptr->jindices[iter] = (unsigned short)myconstraints[myconstraintsindex[i][iter + 2]];
				blockptr->entries[iter] = myconstraintsvalue[myconstraintsindex[i][iter + 2]];
			}
			blockptr->iindices[myconstraintsindex[i][2] + 1] = (unsigned short)(allsize + 1);
			blockptr->jindices[myconstraintsindex[i][2] + 1] = (unsigned short)(allsize + 1);
			blockptr->entries[myconstraintsindex[i][2] + 1] = 1.0;
		}
		else{
			blockptr->numentries = myconstraintsindex[i][2];
			for (int iter = 1; iter <= myconstraintsindex[i][2]; iter++){
				blockptr->iindices[iter] = (unsigned short)myconstraints[myconstraintsindex[i][iter + 2]];
				blockptr->jindices[iter] = (unsigned short)myconstraints[myconstraintsindex[i][iter + 2]];
				blockptr->entries[iter] = myconstraintsvalue[myconstraintsindex[i][iter + 2]];
			}
		}

		blockptr->next = constraints[i].blocks;
		constraints[i].blocks = blockptr;

	}

	constraints[conssize + 1].blocks = NULL;

	blockptr = (struct sparseblock *)malloc(sizeof(struct sparseblock));
	if (blockptr == NULL)
	{
		printf("Allocation of constraint block failed!\n");
		exit(1);
	};

	blockptr->blocknum = 1;
	blockptr->blocksize = allsize + 1;
	blockptr->constraintnum = conssize + 1;
	blockptr->next = NULL;
	blockptr->nextbyblock = NULL;
	blockptr->entries = (double *)malloc((gsize + 2)*sizeof(double));
	if (blockptr->entries == NULL)
	{
		printf("Allocation of constraint block failed!\n");
		exit(1);
	};
	blockptr->iindices = (unsigned short *)malloc((gsize + 2)*sizeof(unsigned short));
	if (blockptr->iindices == NULL)
	{
		printf("Allocation of constraint block failed!\n");
		exit(1);
	};
	blockptr->iindices[0] = (unsigned short)0;
	blockptr->jindices = (unsigned short *)malloc((gsize + 2)*sizeof(unsigned short));
	if (blockptr->jindices == NULL)
	{
		printf("Allocation of constraint block failed!\n");
		exit(1);
	};
	blockptr->jindices[0] = (unsigned short)0;

	blockptr->numentries = gsize + 1;

	for (int j = 1; j <= gsize + 1; j++){
		blockptr->iindices[j] = (unsigned short)(fsize + j);
		blockptr->jindices[j] = (unsigned short)(fsize + j);
		blockptr->entries[j] = 1.0;
	}

	blockptr->next = constraints[conssize + 1].blocks;
	constraints[conssize + 1].blocks = blockptr;



	//write_prob("prob.dat-s", allsize + 2 + varsize, conssize + 1, C, b, constraints);

	initsoln(allsize + 2 + varsize, conssize + 1, C, b, constraints, X, &y, &Z);

	ret = easy_sdp_1(allsize + 2 + varsize, conssize + 1, C, b, constraints, 0.0, X, &y, &Z, &pobj, &dobj);

	//write_sol("prob.sol", allsize + 2 + varsize, conssize + 1, *X, y, Z);

	/*
	* Free storage allocated for the problem and return.
	*/

	//free_prob(allsize + 2 + varsize, conssize + 1, C, b, constraints, *X, y, Z);

	return ret;
}